#include <stdio.h>
#include <iostream.h>
#include <math.h>
#include <nnview/matrix.h>

#define BETA	0.1

//--------------------------------------------------
// network size, # of training input
//--------------------------------------------------
int inpSize, hidSize, outSize;
int numInp;
double eta, mom; 	// learning rate eta. momentum mom (not used).
int biasFlag;		// bias flag : 0 = no bias, 1 = with bias

//--------------------------------------------------
// activities (after activation function)
//--------------------------------------------------
double *input;
double *hidden;
double *output;
double *target;

//--------------------------------------------------
// net input
//--------------------------------------------------
double *netHidden;
double *netOutput;

//--------------------------------------------------
// deltas
//--------------------------------------------------
double *deltaHidden;
double *deltaOutput;

//--------------------------------------------------
// weights : mat->mat[from][to]
//--------------------------------------------------
matrix<double> *input2hidden;
matrix<double> *hidden2output;
matrix<double> *inp2hidMom;
matrix<double> *hid2outMom;

//--------------------------------------------------
// input and target
//--------------------------------------------------
matrix<double> *dataIn;
matrix<double> *dataOut;

//----------------------------------------------------------------------------
// setupNetwork
//----------------------------------------------------------------------------
void setupNetwork (int inpSize, int hidSize, int outSize, int numInp) {

  cerr 	<< "inpSize = " << inpSize << " hidSize = " << hidSize 
  	<< " outSize = " << outSize  << endl;

  input 	= new double[inpSize]; 
  hidden 	= new double[hidSize]; 
  output 	= new double[outSize];
  target	= new double[outSize];

  netHidden 	= new double[hidSize];
  netOutput 	= new double[outSize];

  netHidden 	= new double[hidSize];
  netOutput 	= new double[outSize];

  deltaHidden 	= new double[hidSize];
  deltaOutput 	= new double[outSize];

  input2hidden 	= new matrix<double>(inpSize,hidSize);
  hidden2output = new matrix<double>(hidSize,outSize);

  inp2hidMom 	= new matrix<double>(inpSize,hidSize);
  hid2outMom = new matrix<double>(hidSize,outSize);

  input2hidden->random_matrix(); 
  hidden2output->random_matrix(); 

  dataIn	= new matrix<double>(numInp,inpSize);
  dataOut	= new matrix<double>(numInp,outSize);

}
//----------------------------------------------------------------------------
// printVector
//----------------------------------------------------------------------------
void printVector (double *v, int size) {
  for (int i=0; i<size; ++i) {
	cout << v[i] << " ";
  }
}

//----------------------------------------------------------------------------
// processConfig
//----------------------------------------------------------------------------
void processConfig (char *filename) {
 
  FILE *fp;

  biasFlag = 0; // default is with no bias.
  
  if (!(fp=fopen(filename,"r"))) {
	cerr << "processConfig(): file error " << filename << endl;
	exit(0);
  }

  fscanf(fp,"%d %d %d\n",&inpSize, &hidSize, &outSize);
  fscanf(fp,"%d\n",&numInp);
  fscanf(fp,"%lf %lf %d\n",&eta,&mom,&biasFlag);

  cout << "eta = " << eta << " biasFlag = " << biasFlag << endl;

  // setup network : if biasFlag == 1, turn on bias
  if (biasFlag) {
	inpSize +=1;
	hidSize +=1;
  }
  setupNetwork(inpSize,hidSize,outSize, numInp);

  for (int i=0; i<numInp; ++i) {
	// input
	for (int j=0; j<inpSize-biasFlag; ++j) {
		fscanf(fp,"%lf",&(dataIn->mat[i][j]));
	}
	if (biasFlag) dataIn->mat[i][inpSize-biasFlag]=1.0;
	// output
	for (int j=0; j<outSize; ++j) {
		fscanf(fp,"%lf",&(dataOut->mat[i][j]));
	}
	fscanf(fp,"\n");
  }

  fclose(fp);

}

//----------------------------------------------------------------------------
// actFunc
//----------------------------------------------------------------------------
double actFunc(double x) {
  // return (1.0/(1.0+exp(-2.0*BETA*x)));
  return (tanh(BETA*x));
}

//----------------------------------------------------------------------------
// actFuncDeriv : arg is f(x)
//----------------------------------------------------------------------------
double actFuncDeriv(double fx) {
  // return 2.0*BETA*fx*(1.0-fx);
  return (1-fx*fx);
}

//----------------------------------------------------------------------------
// feedForward
//----------------------------------------------------------------------------
void feedForward(double *inp, matrix<double> *wgt, double *netOut, double *out){
 
  int inpSize = wgt->get_cols();
  int outSize = wgt->get_rows();

  // set bias unit
  if (biasFlag) { 
	inp[inpSize] = 1.0;
  }

  for (int t=0; t<outSize; ++t) {
	netOut[t] = 0.0;
  	for (int i=0; i<inpSize; ++i) {
		netOut[t] += inp[i]*wgt->mat[i][t];
	}
	out[t] = actFunc(netOut[t]);
  }

} 

//----------------------------------------------------------------------------
// feedBack 
//----------------------------------------------------------------------------
void feedBack(double *inp, matrix<double> *wgt, double *actOut, double *out){
 
  int inpSize = wgt->get_rows();
  int outSize = wgt->get_cols();

  for (int t=0; t<outSize; ++t) {
	out[t] = 0.0;
  	for (int i=0; i<inpSize; ++i) {
		out[t] += inp[i]*wgt->mat[t][i];
	}
	out[t] *= actFuncDeriv(actOut[t]);	
  }

} 

//----------------------------------------------------------------------------
//
// main
//
//----------------------------------------------------------------------------
int main (int argc, char *argv[]) {

  if (argc!=2) {
	cerr << "usage: bp <configfile>" << endl;
 	exit(0);
  }

  processConfig(argv[1]); 

  cout << endl;
  cout << "dataIn" << endl ;
  dataIn->print_matrix();
  cout << endl;
  cout << "dataOut" << endl ;
  dataOut->print_matrix();

  int i=0;
  while (1) {

     double err = 0.0, errSum=0.0;
     //-------------------------------------------------- for
     for (int t=0; t<numInp; ++t) {
	// 1. Get training data
	for (int j=0; j<inpSize; ++j) input[j] = dataIn->mat[t][j];
	for (int j=0; j<outSize; ++j) target[j] = dataOut->mat[t][j];
	
	// 2. feedforward
	feedForward(input,input2hidden,netHidden,hidden);
	feedForward(hidden,hidden2output,netOutput,output);

	// 3. get error
	for (int j=0; j<outSize; ++j) {
		err = target[j]-output[j];	
		deltaOutput[j] = actFuncDeriv(output[j])*err;
		errSum += err*err;
	} 
   	// cout << endl;

	// 4. back prop delta
	feedBack(deltaOutput,hidden2output,hidden,deltaHidden);

	// 5. Adapt weight 
	for (int j=0; j<outSize; ++j) 
	   for (int k=0; k<hidSize; ++k) {
		hidden2output->mat[k][j] += eta * deltaOutput[j] * hidden[k];
	   }

	for (int j=0; j<hidSize; ++j) 
	   for (int k=0; k<inpSize; ++k) {
		input2hidden->mat[k][j] += eta * deltaHidden[j] * input[k];
	   }

        if (i%10000==0) {
		printVector(input,inpSize) ; 
		cout << " : out " ; 
		printVector(output,outSize);
		cout << " : target " ; 
		printVector(target,outSize);
		cout << " : delta " ; 
		printVector(deltaOutput,outSize);
		cout << " : deltaHidden " ; 
		printVector(deltaHidden,hidSize);
		cout << endl;
		cout << endl;
		cout << " input2hidden : " ;
		input2hidden->print_matrix();
		cout << " hidden2output : " ;
		hidden2output->print_matrix();
		cout << endl;
		cout << " : hidden ";
		printVector(hidden,hidSize);
		cout << endl;

		cout << "SOL: ";
		printVector(target,outSize);
		cout << " " ;
		printVector(output,outSize);
		cout << endl;
         }
    } // end of one epoch
    //-------------------------------------------------- end for

    ++i;
    if (i%10000==0) {
    	cout << "ERR " << (i/10000) << " " << (errSum/numInp) << endl;
    }
    if (errSum/numInp < 0.01) {
	cerr << "Target err reached." << endl;
	break;
    }

  }

}

