//----------------------------------------------------------------------------
// 
//	file: twoneuron.cc
// 
//	function: test two simple SLISSOM neurons 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Mon Mar 15 04:21:10 CST 1999
// 
//	$Revision: 1.2 $ $Date: 2000/01/30 14:55:55 $
//	$Log: twoneuron.cc,v $
//	Revision 1.2  2000/01/30 14:55:55  yschoe
//	*** empty log message ***
//
//	Revision 1.1  1999/03/16 23:26:47  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------
#include <iostream.h>
#include <nnview/real.h>
#include <nnview/spike-code.h>
#include <stdlib.h>
#include <math.h>

//----------------------------------------------------------------------------
//	main() 
//	  -  main driver
//----------------------------------------------------------------------------
int main (int argc, char *argv[]) {

  if (argc!=18) {
    cout << "usage: twoneuron x1 x2 syn_decay loop_decay alpha_exc " 
	 << "tau iters gamma_exc gamma_aff gamma_inh theta noise " 
         << "MASK settle radius add_to_spike exc_w" << endl;
    exit(1);
  }
  
  // get parameters
  REAL x1 = atof(argv[1]);	// membr. pot.
  REAL x2 = atof(argv[2]);
  
  int  s1 = 0; // spike 
  int  s2 = 0;

  REAL syn_decay = atof(argv[3]);
  REAL loop_decay= atof(argv[4]);
  REAL alpha_exc = atof(argv[5]);
  REAL tau 	= atof(argv[6]);
  int  iters 	= atoi(argv[7]);
  REAL gamma_exc = atof(argv[8]);
  REAL gamma_aff = atof(argv[9]);
  REAL gamma_inh = atof(argv[10]);
  REAL theta	 = atof(argv[11]);
  REAL noise 	 = atof(argv[12]);
  int  MASK	 = atoi(argv[13]);
  int  settle	 = atoi(argv[14]);
  int  radius    = atoi(argv[15]);
  REAL add_to_spike = atof(argv[16]);
  REAL exc_w = atof(argv[17]);

  // suppose self exc and self inhibition cancels out 

  int  qlen = (int)(log(MASK)/log(2))+1;
  
  // cout << qlen << " : Q length" << endl;

  REAL *syn_lut = init_normalized_leaky_table(qlen,syn_decay);
  REAL *loop_lut = init_normalized_leaky_table(qlen,loop_decay);

  REAL glob_inh = 0.0; // must be the sum of activity

  for (int i=0; i<iters; ++i) {

      int spike1, spike2;
      REAL i1, i2;
      REAL thresh1, thresh2;
      REAL v1, v2;

      // net input
      REAL excw = (radius==1) ? exc_w : 1.0;
      REAL mult = (radius==1) ? 1.0 : 0.0;

      i1 = i2 = 1.0; // (sin(M_PI*i/180.0)+1.0)/2.0;

      for (int n=0; n<settle; ++n) {
      x1 = gamma_aff * i1 		// input
	 + gamma_exc * excw * mult * syn_lut[s2&0x7] 	// exc
	 + gamma_exc * excw * syn_lut[s1&0x7] 	// self exc
	 - gamma_inh * glob_inh ;
      x1 = (1-noise)*x1+noise*drand48();

      x2 = gamma_aff * i2 		// input
	 + gamma_exc * excw * mult * syn_lut[s1&0x7] 	// exc
	 + gamma_exc * excw * syn_lut[s2&0x7] 	// self exc
	 - gamma_inh * glob_inh ;
      x2 = (1-noise)*x2+noise*drand48();

      // spike generation
      thresh1 = theta+tau*loop_lut[s1&MASK];
      spike1 = (x1-thresh1>0.0)?1:0;
      v1 = x1-thresh1+theta;
      s1 = s1<<1 | spike1;
  	
      thresh2 = theta+tau*loop_lut[s2&MASK];
      spike2 = (x2-thresh2>0.0)?1:0;
      v2 = x2-thresh2+theta;
      s2 = s2<<1 | spike2;

      cout << "intr" << (i+n*(1.0/(REAL)(settle+2))+iters)  << " " << x1 << " " << x2 << " " 
	   << (spike1) << " " << (spike2+add_to_spike) << " " 
	   << glob_inh << " " 
	   << (syn_lut[s1&MASK]) << " " << (syn_lut[s2&MASK]) << " "
	   << thresh1 << " " << thresh2 << " "
	   << v1 << " " << v2 << endl;
      

      }

      glob_inh = (syn_lut[s2&0x7]+syn_lut[s1&0x7]);

      cout << i  << " " << x1 << " " << x2 << " " 
	   << (spike1) << " " << (spike2+add_to_spike) << " " 
	   << glob_inh << " " 
	   << (syn_lut[s1&MASK]) << " " << (syn_lut[s2&MASK]) << " "
	   << thresh1 << " " << thresh2 << " "
	   << v1 << " " << v2 << " "
	   << theta << endl;
	    
      // no weight mod because there's only one weight and it is
      // normalized, so it is always 1.0
  	
  }

}

