## usage: [Kmax,sKmax,Eimg] = convsigmaplot(Img, theta, freq)
##
##	Img: p x q matrix (grayscale image)
##
##		Img: 2D matrix with grayscale image
##		theta: gabor filter orientation 
##              freq: spatial frequency of the convolution kernel
##
## First, convolve with DoG filter, and then convolve with gabor
## filters of kernel sizes 3,5,7,11,13,15,17.
##
##	Usage example:
##		[Kmax,sKmax,Eimg] = convsigmaplot(imagemat,0,1);
##
##		Gabor filter orientation is 0.
##		Gabor filter frequency is 1.
##
## $Id: convedgemat.m,v 1.2 2002/03/09 17:31:27 yschoe Exp yschoe $

function [Kmax,sKmax,Eimg] = convsigmaplot(Img, theta, freq)

	# get image size
	[oldp, oldq] = size(Img);

	# adjust grid size to have the conv kernels
	# centered at the middle: i.e. make the gridsize an odd number.
	#if ( rem(kernelsize,2) == 0 ) 
	#	kernelsize ++;
	#endif
        kernelsize=7;
	DoG = dog(kernelsize);

        # convolve with DoG, and only use the real part
	Eimg = real(conv2(Img,DoG,"same"));	
  
       	sKmax=zeros(oldp,oldq);
        Kmax=zeros(oldp,oldq);
       
        kernelsize=3;
	g = gabor(kernelsize,kernelsize,"sin",theta,freq);

        sKmax=real(conv2(Eimg,g,"same"));
	Kmax=Kmax+1;
        i=1;

        #find sKmax and Kmax iteratively
	while (kernelsize<=17)
             g = gabor(kernelsize,kernelsize,"sin",theta,freq);
             sKmax=max(sKmax,real(conv2(Eimg,g,"same")));
             Kmax=max(Kmax,(i+1)*(sign((real(conv2(Eimg,g,"same")))-sKmax)+1));
             i=i+1;
             kernelsize=kernelsize+2;
        endwhile


endfunction
