#!/usr/bin/wish

global cog_size retina_size cog cog2 rf_radius

#----------------------------------------------------------------------------
# 	fix: 	limit 'idx' between [0..$size-1]
#----------------------------------------------------------------------------
proc fix { idx size } {

  if { $idx < 0 } {
	return 0;
  }

  if { $idx >= $size } {
	set ret [expr $size - 1 ];
	return $ret
  } else {
	return $idx
  }
}
#----------------------------------------------------------------------------
# 	plotcontelem: plot contour elements
#----------------------------------------------------------------------------
proc plotcontelem { cur mode {count 0} } { 

  # the argument "cur" is in the form : <x>,<y>,<theta>
  # where 
  #	<x> 	: x location on map
  #	<y>	: y location on map
  #  	<theta> : orientation preference at x,y in degrees
  # plot the input segments in the COG window

  global cog_size retina_size rf_radius cog cog2 sigmax sigmay
  global line line2


  set tagflag 0

  # erase previous step's current location marker
  clearcontelem "move"

  # change color coding depending on "click" or "move" modes.
  # "click" mode -- when it is clicked and the location needs to be recorded.
  # "move" mode -- when moving. Just display the current location and
  #  	remove previous traces.
  if { $mode == "click" } {
	set inpcolor "red"
	set rfcolor "blue"
  } elseif { $mode == "background"} {
	set inpcolor \#aaaaff
	set rfcolor \#ffaaaa
	set tagflag 1
  } else {
	set inpcolor "blue"
	set rfcolor "red"
  }

  #----------------------------------------
  # Get cortex location x, y and orientation preference theta. 
  #----------------------------------------
  set curlist [split $cur ","]

  set x [lindex $curlist 0]
  set y [lindex $curlist 1]
  set theta [lindex $curlist 2]
  set rad [expr ($theta)*3.141592/180.0]
  set orthorad [expr ($theta+180)*3.141592/180.0]

  # cx : RF x center in retinal space
  # cy : RF y center in retinal space
  set c $cog($x-$y)
  set cl [split $c ":"]
  set cx [expr int([lindex $cl 0])]
  set cy [expr -int([lindex $cl 1])+$retina_size-1]

  #----------------------------------------
  # Calculate contour segment coordinates and plot it : .misc.inp canvas
  #----------------------------------------
  set gran  [expr $cog_size/$retina_size]
  set midx  [expr int($cx*$gran)]
  set midy  [expr int($cy*$gran)]
  set srcx [expr int(cos($orthorad)*$sigmay*$gran/2.0)+$midx]
  set srcy [expr -int(sin($orthorad)*$sigmay*$gran/2.0)+$midy-$gran]
  set destx [expr int(cos($rad)*$sigmay*$gran)+$midx]
  set desty [expr -int(sin($rad)*$sigmay*$gran)+$midy-$gran]

  .misc.inp create line\
		$srcx $srcy $destx $desty \
		-fill $inpcolor -width [expr 2*$sigmax*$gran] \
		-capstyle round -tags [list $mode]

  if {$tagflag} {
  	.misc.inp create text $srcx $srcy -text "$count" -tags [list $mode]
  }

  #----------------------------------------
  # plot approximate RF of the unit drawn above 
  #----------------------------------------
  set srcx [fix [expr ($midx-$rf_radius*$gran)] $cog_size]
  set srcy [fix [expr ($midy-$rf_radius*$gran)] $cog_size]
  set destx [fix [expr ($midx+$rf_radius*$gran)] $cog_size]
  set desty [fix [expr ($midy+$rf_radius*$gran)] $cog_size]

  .misc.inp create oval \
		  $srcx $srcy $destx $desty \
		  -outline $rfcolor -tags [list $mode]
  
  update

}

#----------------------------------------------------------------------------
# 	clearcontelem: clear all contour elements
#----------------------------------------------------------------------------
proc clearcontelem { mode } { 

  # clear existing labels
  .misc.inp addtag allItems withtag $mode
  .misc.inp delete allItems

}

#----------------------------------------------------------------------------
# 	plotcog: plot c.o.g.s in image COG
#----------------------------------------------------------------------------
proc plotcog { x y x2 y2 or or2 mode} {

  global cog_size retina_size rf_radius cog cog2
  global line line2

  #----------------------------------------
  # Plot center of gravity : as circle with cross-shaped hair
  # 	in net display window
  #----------------------------------------

  set gran [expr int($cog_size/$retina_size)] 
  if { $gran==0 } {
	set gran 1
  }

  # calculate coordinates
  set c $cog($x-$y)
  set cl [split $c ":"]
  set cx [expr int([lindex $cl 0])]
  set cy [expr -int([lindex $cl 1])+$retina_size-1]

  set px [fix [expr ($cx-$rf_radius)] $retina_size]
  set py [fix [expr ($cy-$rf_radius)] $retina_size]

  set c2 $cog2($x2-$y2)
  set cl2 [split $c2 ":"]
  set cx2 [expr int([lindex $cl2 0])]
  set cy2 [expr -int([lindex $cl2 1])+$retina_size-1]

  # remove old plots (circle with cross-shaped hair)
  .misc.net addtag allOvals withtag ovals
  .misc.net delete allOvals

  # Now create new RF plots for map1
  .misc.net create oval \
		[expr $px*$gran] \
		[expr $py*$gran] \
		[expr ($cx+$rf_radius)*$gran] \
		[expr ($cy+$rf_radius)*$gran] \
		  -outline red -tags [list ovals]

  .misc.net create line \
		[expr $cx*$gran] [expr $py*$gran] \
		[expr $cx*$gran] [expr ($cy+$rf_radius)*$gran] \
	-fill red -width 1 -capstyle round -tags [list $mode]

  .misc.net create line \
		[expr $px*$gran] [expr $cy*$gran] \
		[expr ($cx+$rf_radius)*$gran] [expr $cy*$gran] \
	-fill red -width 1 -capstyle round  -tags [list $mode]

  set px [fix [expr ($cx2-$rf_radius)] $retina_size]
  set py [fix [expr ($cy2-$rf_radius)] $retina_size]

  # create new RF plots for map2
  .misc.net create oval \
		[expr $px*$gran] \
		[expr $py*$gran] \
		[expr ($cx2+$rf_radius)*$gran] \
		[expr ($cy2+$rf_radius)*$gran] \
		  -outline blue -tags [list ovals]

  .misc.net create line \
		[expr $cx2*$gran] [expr $py*$gran] \
		[expr $cx2*$gran] [expr ($cy2+$rf_radius)*$gran] \
	-fill blue -width 1 -capstyle round -tags [list $mode]

  .misc.net create line \
		[expr $px*$gran] [expr $cy2*$gran] \
		[expr ($cx2+$rf_radius)*$gran] [expr $cy2*$gran] \
	-fill blue -width 1 -capstyle round  -tags [list $mode]


  update

  #----------------------------------------
  # plot oriented RF from c.o.g. coordinates and given orientations
  #----------------------------------------
  
  # clear existing lines
  .misc.ori addtag allLines all
  .misc.ori delete allLines
 
  # Draw outline circle, and 0 and 90 degrees
  .misc.ori create oval 4 4 $cog_size $cog_size \
	-fill white -outline black -width 2 

  set midx  [expr int($cog_size/2)]
  set midy  [expr int($cog_size/2)]

  .misc.ori create line $midx 0 $midx $cog_size \
	 -fill \#cccccc -width 1 
  .misc.ori create line 0 $midy $cog_size $midy \
	 -fill \#cccccc -width 1 

  #--------------------
  # plot orientation preference of first map
  #--------------------
  set sizex [expr int($cog_size/2.3)]
  set sizey [expr int($cog_size/2.3)]

  set diffx [expr ($cx-$cx2)*5]
  set diffy [expr ($cy-$cy2)*5]

  set rad [expr ($or)*3.141592/180.0]
  set rad2  [expr ($or2)*3.141592/180.0]
  set orthorad [expr ($or+180)*3.141592/180.0]
  set orthorad2  [expr ($or2+180)*3.141592/180.0]

  set midx  [expr int($cog_size/2)]
  set midy  [expr int($cog_size/2)]
  set srcx [expr int(cos($orthorad)*$sizex)+$midx]
  set srcy [expr int(sin($orthorad)*$sizey)+$midy]
  set destx [expr int(cos($rad)*$sizex)+$midx]
  set desty [expr int(sin($rad)*$sizey)+$midy]

  .misc.ori create line $srcx [expr $cog_size-$srcy-1] \
	 $destx [expr $cog_size-$desty-1] \
	 -fill red -width 10 \
	-arrow both -capstyle round

  .misc.ori create oval [expr $midx-9] [expr $midy-9] \
		[expr $midx+9] [expr $midy+9]  -fill red -outline red

  .misc.ori create oval [expr $midx-6] [expr $midy-6] \
		[expr $midx+6] [expr $midy+6]  -fill white -outline white

  
  #--------------------
  # plot orientation preference of second map
  #--------------------
  set midx2 [expr $midx+$diffx]
  set midy2 [expr $midy+$diffy]
  set srcx2 [expr int(cos($orthorad2)*$sizex)+$midx2]
  set srcy2 [expr int(sin($orthorad2)*$sizey)+$midy2]
  set destx2 [expr int(cos($rad2)*$sizex)+$midx2]
  set desty2 [expr int(sin($rad2)*$sizey)+$midy2]

  .misc.ori create line $srcx2 [expr $cog_size-$srcy2-1] \
	$destx2 [expr $cog_size-$desty2-1] \
	 -fill blue -width 5 \
	-arrow both -capstyle round

  .misc.ori create oval [expr $midx2-6] [expr $cog_size-($midy2-6)-1] \
		[expr $midx2+6] [expr $cog_size-($midy2+6)-1] \
		 -fill blue -outline blue
  
  #--------------------
  # Draw scale
  #--------------------
  .misc.ori create line [expr $cog_size/10] [expr $cog_size/10] \
	[expr $cog_size/10 + 5] [expr $cog_size/10] \
	-fill black -width 3 \
	-capstyle round

}

#----------------------------------------------------------------------------
# 	plotnet: plot c.o.g.s net 
#----------------------------------------------------------------------------
proc plotnet { size } {

  global cog_size retina_size rf_radius cog cog2
  global line line2

  set gran [expr int($cog_size/$retina_size)] 
  if { $gran==0 } {
	set gran 1
  }
  #--------------------
  # clear existing lines
  #--------------------
  .misc.net addtag allLines all
  .misc.net delete allLines
	 
  for {set x 0} {$x<[expr $size-1]} {incr x} {
     for {set y 0} {$y<[expr $size-1]} {incr y} {

  	  #--------------------
	  # current node
  	  #--------------------
	  set c $cog($x-$y)
	  set cl [split $c ":"]
	  set cx [expr [lindex $cl 0]]
	  set cy [expr -[lindex $cl 1]+$retina_size-1]

  	  #--------------------
	  # x neighbor
  	  #--------------------
	  set xplus [expr $x+1]
	  set nx $cog($xplus-$y)
	  set nxl [split $nx ":"]
	  set nxx [expr [lindex $nxl 0]]
	  set nxy [expr -[lindex $nxl 1]+$retina_size-1]

  	  #--------------------
	  # y neighbor
  	  #--------------------
	  set yplus [expr $y+1]
	  set ny $cog($x-$yplus)
	  set nyl [split $ny ":"]
	  set nyx [expr [lindex $nyl 0]]
	  set nyy [expr -[lindex $nyl 1]+$retina_size-1]

	  .misc.net create line [expr int($cx*$gran)] [expr int($cy*$gran)] \
		[expr int($nxx*$gran)] [expr int($nxy*$gran)] \
		 -fill \#000000 -width 1 
	  .misc.net create line [expr int($cx*$gran)] [expr int($cy*$gran)] \
		[expr int($nyx*$gran)] [expr int($nyy*$gran)] \
		 -fill \#000000 -width 1 
    }
  }
}

#----------------------------------------------------------------------------
#	parse_and_add
#----------------------------------------------------------------------------
proc parse_and_add { input_config_file map_num or cog } {

  upvar $or or_list
  upvar $cog cog_list

  puts stderr "*** parse_and_add $input_config_file $map_num"

  # Extract coordinates
  set fp [open $input_config_file]
  if { $map_num == 1 } {
  	append pattern_str "call_add_mua_spec_circle " \
		{ ([0-9]+) ([0-9]+) }
  } else {
  	append pattern_str "call_add_mua_spec_circle$map_num" \
		{ ([0-9]+) ([0-9]+) }
  }
  set count 1
  while { ![eof $fp] } {
  
    set line [gets $fp]
	
    if { [regexp $pattern_str $line whole x y] } {
		puts stderr " * MATCH: x=$x, y=$y, ori=$or_list($x-$y)"
		set cur "$x,$y,$or_list($x-$y)"
		set curcog [split $cog_list($x-$y) ":"];\
         	set cogx [lindex $curcog 0];\
         	set cogy [lindex $curcog 1];\
		plotcontelem $cur "background" $count; \
		incr count
		update
         	# puts -nonewline :$cur,$cogx,$cogy; \
    }
    
  }
  close $fp

}

#----------------------------------------------------------------------------
#
# 	MAIN:
#
#----------------------------------------------------------------------------
if {[llength $argv] <= 0} {
  puts "usage: printcoord2.tcl <size> <zoom> <ppm_filename> <or_filename> "
  puts "	<size2> <ppm_filename2> <or_filename2>"
  puts " 	<cog_filename1> <cog_filename2> <retina_size> <rfradius>"
  puts " 	[<simul.conf>]"
  exit
}

#----------------------------------------
# 1.1 Load image file : ppm or gif
#----------------------------------------
set size [lindex $argv 0]
set zoom [lindex $argv 1]
image create photo MAP -file "[lindex $argv 2]"
image create photo ZoomMAP 
ZoomMAP copy MAP -zoom $zoom $zoom

#----------------------------------------
# 1.2 Load image file2 : ppm or gif
#----------------------------------------
set size2 [lindex $argv 4]
image create photo MAP2 -file "[lindex $argv 5]"
image create photo ZoomMAP2 
set zoom2 [expr $size*$zoom/$size2]
ZoomMAP2 copy MAP2 -zoom $zoom2 $zoom2

#----------------------------------------
# 2.1 Load orientation
#----------------------------------------
set fp [open [lindex $argv 3]]
for {set i 0} {$i<$size} {incr i} {

  set line [string trim [gets $fp] " "]
  set cols [split $line " "]
  for {set j 0} {$j<=[expr [llength $cols]-1]} {incr j} {
	set or(${j}-${i}) [expr [lindex $cols $j]*180.0]
  }
}
close $fp

#----------------------------------------
# 2.2 Load orientation2
#----------------------------------------
set fp [open [lindex $argv 6]]
for {set i 0} {$i<$size2} {incr i} {

  set line [string trim [gets $fp] " "]
  set cols [split $line " "]
  for {set j 0} {$j<=[expr [llength $cols]-1]} {incr j} {
	set or2(${j}-${i}) [expr [lindex $cols $j]*180.0]
  }
}
close $fp
#----------------------------------------
# 3.1 Load COG1 : each loaded cog items are in the format of "x:y"
#----------------------------------------
set fp [open [lindex $argv 7]]
for {set i 0} {$i<$size} {incr i} {

  set line [string trim [gets $fp] " "]
  set cols [split $line " "]
  for {set j 0} {$j<=[expr [llength $cols]-1]} {incr j} {
	set cog(${j}-${i}) [lindex $cols $j]
	# regsub ?switches? exp string subSpec varName
	regsub "nan:nan" $cog(${j}-${i}) "0.0:0.0" cog(${j}-${i})
  }

}
close $fp


#----------------------------------------
# 3.2 Load COG2
#----------------------------------------
set fp [open [lindex $argv 8]]
for {set i 0} {$i<$size2} {incr i} {

  set line [string trim [gets $fp] " "]
  set cols [split $line " "]
  for {set j 0} {$j<=[expr [llength $cols]-1]} {incr j} {
	set cog2(${j}-${i}) [lindex $cols $j]
  }

}
close $fp

set retina_size [lindex $argv 9]
if {$retina_size<30} {
	set cog_size 125
} else {
	set cog_size [expr $retina_size * 4]
}
set rf_radius [lindex $argv 10]

#----------------------------------------
# 3.3 If the optional config file argument is given, read in 
# 	input points from the file
#----------------------------------------

	# This needs to be done after the graphics has been
	# initialize to ensure that the loaded inputs 
	# are displayed properly : see section 8.

# image create photo COG -width $cog_size -height  $cog_size

#----------------------------------------
# 4.1 Create display window label
#----------------------------------------
frame .mapframe -relief sunken -borderwidth 5 \
	-background black
pack .mapframe -padx 10 -pady 10 -side top
label .mapframe.map1 -image ZoomMAP
pack .mapframe.map1 -side left -padx 10 -pady 10
label .mapframe.map2 -image ZoomMAP2
pack .mapframe.map2 -side left -padx 10 -pady 10


#----------------------------------------
# 4.2 Create ormap frame : beware.. this is UGLY! 
#----------------------------------------
set datstr {[expr %x/$zoom],[expr $size-%y/$zoom-1]}
set color red
bind .mapframe.map1 <Button-1> { \
	 set xc [expr %x/$zoom]; \
	 set yc [expr $size-%y/$zoom-1]; \
	 set xc [fix $xc $size]; \
	 set yc [fix $yc $size]; \
	 set yc_screen [expr %y/$zoom]; \
	 set cur $xc,$yc,$or($xc-$yc); \
	 ZoomMAP put \#000000 -to [expr $xc*$zoom] \
		[expr $yc_screen*$zoom] \
		[expr $xc*$zoom + $zoom] \
		[expr $yc_screen*$zoom+$zoom]; \
	 \
	 set xc2 [expr $xc*$size2/$size]; \
	 set yc2 [expr $yc*$size2/$size]; \
	 set xc2 [fix $xc2 $size2]; \
	 set yc2 [fix $yc2 $size2]; \
	 set yc2_screen [expr $yc_screen*$size2/$size]; \
	 set cur2 $xc2,$yc2,$or2($xc2-$yc2);\
	 ZoomMAP2 put \#000000 -to [expr $xc2*$zoom2] \
		[expr $yc2_screen*$zoom2] \
		[expr $xc2*$zoom2 + $zoom2] \
		[expr $yc2_screen*$zoom2+$zoom2]; \
	 plotcog $xc $yc $xc2 $yc2 $or($xc-$yc) $or2($xc2-$yc2) "permovals";\
	 set curcog [split $cog($xc-$yc) ":"];\
	 set cogx [lindex $curcog 0];\
	 set cogy [lindex $curcog 1];\
	 update; puts -nonewline :$cur,$cogx,$cogy; \
	 plotcontelem $cur "click";\
	 flush stdout}
bind .mapframe.map1 <Motion> \
	{  \
	   set xc [expr %x/$zoom]; \
	   set yc [expr $size-%y/$zoom-1]; \
	   set xc [fix $xc $size]; \
	   set yc [fix $yc $size]; \
	   set cur $xc,$yc,$or($xc-$yc); \
	   set map1_or $or($xc-$yc); \
	   \
	   set xc2 [expr $xc*$size2/$size]; \
	   set yc2 [expr $yc*$size2/$size]; \
	   set xc2 [fix $xc2 $size2]; \
	   set yc2 [fix $yc2 $size2]; \
	   set cur2 $xc2,$yc2,$or2($xc2-$yc2);\
	   if { $map1_or == $or2($xc2-$yc2) } { 
		set color green 
	   } else { 
		set color red 
	   };\
	   .coord configure -foreground $color; \
	   .coord2 configure -foreground $color; \
	   plotcog $xc $yc $xc2 $yc2 $or($xc-$yc) $or2($xc2-$yc2) "ovals";\
	   plotcontelem $cur "move";\
	   update; \
	}

#----------------------------------------
# 4.3.1 Create Coordinate display window
#----------------------------------------
label .coord -textvariable cur -relief sunken \
	-foreground red -background black -font lucidasans-24 -width 30
set cur "Coordinates"
pack .coord -side top -padx 20 -pady 10 

#----------------------------------------
# 4.3.2 Create Coordinate display window2
#----------------------------------------
label .coord2 -textvariable cur2 -relief sunken \
	-foreground red -background black -font lucidasans-24 -width 30
set cur2 "Coordinates2"
pack .coord2 -side top -padx 20 -pady 10 

#----------------------------------------
# 5. Create COG / Orientation display frame
#----------------------------------------
frame .misc -relief ridge; pack .misc -padx 10 -pady 10

canvas .misc.inp -relief sunken -borderwidth 1 -background white \
	-width $cog_size -height $cog_size; 
pack .misc.inp -side left -padx 10 -pady 5 ; 

canvas .misc.net -relief sunken -borderwidth 1 -background white \
	-width $cog_size -height $cog_size;
pack .misc.net -side left -padx 10 -pady 5 ; 
plotnet $size

canvas .misc.ori -relief sunken -borderwidth 1 -background white \
	-width $cog_size -height $cog_size; 
pack .misc.ori -side left -padx 10 -pady 5 ; 

#----------------------------------------
# 6. Add contour element size windows
#----------------------------------------
frame .param 
pack .param -side top -padx 10 -pady 5;

# default 
set sigmax 1.5
set sigmay 3.5

label .param.xname -text "sigma_x:" 
pack .param.xname -side left
entry .param.x -textvariable sigmax -background white
pack .param.x -side left -padx 10 

label .param.yname -text "sigma_y:" 
pack .param.yname -side left
entry .param.y -textvariable sigmay -background white
pack .param.y -side left -padx 10 

#----------------------------------------
# 7. Add clear and quit button
#----------------------------------------
frame .buttons
pack .buttons -side top -padx 5 -pady 5 

button .buttons.psdump -command { \
  	clearcontelem "move"; \
	.misc.inp postscript -file ormapview.input.ps ; \
	# .misc.net postscript -file ormapview.net.ps ; \
	} -text "PS Dump"
pack .buttons.psdump -side left
	
button .buttons.clear -command { puts "\n* cleared *";  \
	ZoomMAP copy MAP -zoom $zoom $zoom; \
	ZoomMAP2 copy MAP2 -zoom $zoom2 $zoom2; \
	clearcontelem "click" \
	} -text "Clear"
pack .buttons.clear -side left -padx 10 -pady 10

button .buttons.quit -command { puts " "; exit } -text "Quit"
pack .buttons.quit -side left -padx 10 -pady 10

#----------------------------------------
# 8. Parse and load input from other config file
#----------------------------------------
if { $argc == 12 } {
	puts stderr "** printcoord2: loading inputs from Load config file"
	# check if coordinates need to be extracted from 
	# map1 or map2
	# This is a sloppy test: if size > size2, we load
	# map1. if size < size2, we load map2 coordinates from the
	# conf file.	
	set input_conf_file [lindex $argv 11]
	if { $size > $size2 } {
		puts stderr "* Loading MAP1 inputs: $input_conf_file"
		parse_and_add	$input_conf_file 1 or cog
	} else {
		puts stderr "* Loading MAP2 inputs: $input_conf_file"
		parse_and_add	$input_conf_file 2 or cog
	}
}

