## Copyright (C) 2000 uschoe
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2, or (at your option)
## any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, write to the Free
## Software Foundation, 59 Temple Place - Suite 330, Boston, MA
## 02111-1307, USA.

## usage: smooth_thresh(X,window_size,percent)
##
## make all cells in matrix X below threshold 'th' to 0.0.
##

## Author: Yoonsuck Choe <yschoe@cs.utexas.edu>

function [T] = smooth_thresh (X,window_size,percent)

	[r,c] = size(X);

	T = zeros(r,c).+min(vec(X));
	for i=1:window_size:r
		for j=1:window_size:c
			er = min(i+window_size,r);
			ec = min(j+window_size,c);
			th = topnpercent(X(i:er,j:ec),percent);
			for x=i:er
				for y=j:ec
					if (X(x,y)>th)
						T(x,y)=X(x,y);
					endif
				endfor
			endfor
		endfor
	endfor

endfunction

	

